/* debug.c: a SCSI protocol dumper called from do_scsi_command
 * Copyright (C) 1996 Michael K. Johnson
 * 
 * I can be reached via email at johnsonm@redhat.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 */

#include "debug.h"

typedef struct {
  int size;
  char *name;
} cmnds;

cmnds cds[] = {
  {6, "TEST_UNIT_READY"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "INQUIRY"},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "RESERVE_UNIT"},
  {6, "RELEASE_UNIT"},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "SCAN"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "SET_WINDOW"},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "READ"},
  {0, ""},
  {10, "SEND"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "OBJECT_POSITION"},
  {0, ""},
  {0, ""},
  {10, "GET_DATA_BUFFER_STATUS"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "WRITE_BUFFER"},
  {0, ""},
  {0, ""},
};

void dump_send (char *b, int len, FILE *f) {
  unsigned char *buf = (unsigned char *) b;
  int command;
  int i, j=0;

  fprintf(f, "\nDATA OUT:\n");
  command = (int) buf[0];
  if (command > (sizeof(cds)/sizeof(cmnds)) || cds[command].size == 0) {
    fprintf(f, "Command unknown: %d\n", command);
  } else {
    for (i = 0; i < cds[command].size; i++) {
      /* Print a byte */
      fprintf(f, "%2.2x ", buf[i]);
    }
    /* Now print name of command */
    fprintf(f, " Command: %s\n", cds[command].name);
  }
  if (i < len) {
    /* Only if there is data to print, print data-out phase */
    for (j=1 ; i < len; i++, j++) {
      fprintf(f, "%2.2x %s", buf[i], (j==8)?" ":"");
      if (j >= 16) {
	j = 0;
	fprintf(f, "\n");
      }
    }
    /* Finish off with a newline unless one was just printed */
    if (j) {
      fprintf(f, "\n");
    }
  }
  fflush(f);
}

void dump_receive (char *b, int len, unsigned int res, FILE *f) {
  unsigned char *buf = (unsigned char *) b;
  int i, j;

  
  /* just dump data-in */
  fprintf(f, "DATA IN:\nResponse code: %2.2x\n", res);
  for (i=0, j=1 ; i < len; i++, j++) {
    fprintf(f, "%2.2x %s", buf[i], (j==8)?" ":"");
    if (j >= 16) {
      j = 0;
      fprintf(f, "\n");
    }
  }
  /* Finish off with a newline unless one was just printed */
  if (j) {
    fprintf(f, "\n");
  }
  fflush(f);
}
