/* scsicmd.h: header file for generic SCSI command code
 * Copyright (C) 1996 Michael K. Johnson
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 */
#ifndef UMAX_SCSICMD_H
#define UMAX_SCSICMD_H

/* This file requires that linux/../../drivers/scsi/sg.h be included
 * first, because of several data structures.
 */

extern FILE *scsi_debug_out;

#define SCSI_OFF sizeof(struct sg_header)

/* size is NULL to ignore */
void scsi_init(int fd, int *size);

void scsi_init_debug(FILE *f);

int do_scsi_cmd(unsigned in_size,      /* input data size */
		unsigned char *ibuf,   /* input buffer */
		unsigned out_size,     /* size of output buffer */
		unsigned char *obuf,   /* output buffer */
		int      expected      /* expected result, -1 for don't care */
		);


#ifdef  SG_BIG_BUFF
#define BUFSIZE SG_BIG_BUFF
#else
#define BUFSIZE 4096
#endif

/* memcpy with an offset */
#define scsicpy(dest, src, len) memcpy((char *)dest + SCSI_OFF, src, len)

/* normal do_scsi_command call wrapper */
#define docmd(insize, inbuf, outsize, obuf, expected)                        \
  if ((do_scsi_cmd(insize, inbuf, outsize, obuf, expected) != expected)      \
      && expected != -1) {                                                   \
    return -1;                                                               \
  }

/* Often, we do this all at once... */
#define fullcmd(inbuf, scsicname, outsize, outbuf, expected)                 \
  scsicpy(inbuf, (scsicname).cmd, (scsicname).size);                         \
  docmd((scsicname).size, inbuf, outsize, outbuf, expected);

/* I'm using functions derived from Eric Youngdale's scsiinfo
 * program here for dealing with parts of SCSI commands.
 */
static inline void setbitfield(unsigned char * pageaddr, int mask, int shift, int val) {
  *pageaddr = (*pageaddr & ~(mask << shift)) | ((val & mask) << shift);
}

static inline void resetbitfield(unsigned char * pageaddr, int mask, int shift, int val) {
  *pageaddr = (*pageaddr & ~(mask << shift)) | (((!val) & mask) << shift);
}

static inline int getbitfield(unsigned char * pageaddr, int mask, int shift) {
  return ((*pageaddr >> shift) & mask);
}

static inline int getnbyte(unsigned char * pnt, int nbytes) {
  unsigned int result = 0;
  int i;

#ifdef DEBUG
  assert(nbytes < 5);
#endif
  for(i=0; i<nbytes; i++)
    result = (result << 8) | (pnt[i] & 0xff);
  return result;
}

static inline void putnbyte(unsigned char * pnt, unsigned int value, unsigned int nbytes) {
  int i;

#ifdef DEBUG
  assert(nbytes < 5);
#endif
  for(i=nbytes-1; i>= 0; i--){
    pnt[i] = value & 0xff;
    value = value >> 8;
  }
}

#endif /* UMAX_SCSICMD_H */
