/* umax.h: header file for UMAX scanner driver.
 * Copyright (C) 1995 Michael K. Johnson
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 */

#ifndef UMAX_UMAX_H
#define UMAX_UMAX_H

/* Not all of these are defined in scsi.h, so we'll make sure
 * we agree about them here...
 */
#define TEST_UNIT_READY         0x00
#define REQUEST_SENSE           0x03
#define INQUIRY                 0x12
#define RESERVE_UNIT            0x16
#define RELEASE_UNIT            0x17
#define SCAN                    0x1B
#define SET_WINDOW              0x24
#define READ                    0x28
#define SEND                    0x2A
#define OBJECT_POSITION         0x31
#define GET_DATA_BUFFER_STATUS  0x34
#undef  WRITE_BUFFER	/* correct size write_buffer for scanner */
#define WRITE_BUFFER            0x3B

/* SCSI commands */

typedef struct {
  char *cmd;
  int size;
} scsiblk;

static unsigned char inquiryC[] = { INQUIRY, 0x00, 0x02, 0x00, 0xff, 0x00 };
static scsiblk inquiry = { inquiryC, sizeof(inquiryC) };
#define inquiry_return_size 0x94
#define get_inquiry_periph_qual(in) getbitfield(in, 0x07, 5)
#  define IN_periph_qual_lun   0x00
#  define IN_periph_qual_nolun 0x03
#define get_inquiry_periph_devtype(in) getbitfield(in, 0x0a, 0)
#  define IN_periph_devtype_scanner 0x06
#  define IN_periph_devtype_unknown 0x1f
#define get_inquiry_CBHS(in) getbitfield(in + 0x01, 0x03, 2)
#  define IN_CBHS_50   0x00
#  define IN_CBHS_255  0x01
#  define IN_CBHS_auto 0x02
#define get_inquiry_translamp(in) getbitfield(in + 0x01, 0x01, 2)
#define get_inquiry_transavail(in) getbitfield(in + 0x01, 0x01, 1)
#define get_inquiry_scanmode(in) getbitfield(in + 0x01, 0x01, 0)
#  define IN_flatbed 0x00
#  define IN_adf     0x01
#define get_inquiry_response_format(in) getbitfield(in + 0x03, 0x07, 0)
#  define IN_recognized 0x02
#define get_inquiry_vendor(in, buf) strncpy(buf, in + 0x08, 0x08)
#define get_inquiry_product(in, buf) strncpy(buf, in + 0x10, 0x00f)
#define get_inquiry_version(in, buf) strncpy(buf, in + 0x20, 0x04)
#define get_inquiry_fw_quality(in)       getbitfield(in + 0x24, 1, 0)
#define get_inquiry_fw_fast_preview(in)  getbitfield(in + 0x24, 1, 1)
#define get_inquiry_fw_shadow_comp(in)   getbitfield(in + 0x24, 1, 2)
#define get_inquiry_fw_reselection(in)   getbitfield(in + 0x24, 1, 3)
#define get_inquiry_fw_lamp_int_cont(in) getbitfield(in + 0x24, 1, 4)
#define get_inquiry_fw_batch_scan(in)    getbitfield(in + 0x24, 1, 5)
#define get_inquiry_fw_calibration(in)   getbitfield(in + 0x24, 1, 6)
#define get_inquiry_sc_three_pass_color(in)    getbitfield(in + 0x60, 1, 0)
#define get_inquiry_sc_one_pass_color(in)      getbitfield(in + 0x60, 1, 1)
#define get_inquiry_sc_lineart(in)             getbitfield(in + 0x60, 1, 2)
#define get_inquiry_sc_halftone(in)            getbitfield(in + 0x60, 1, 3)
#define get_inquiry_sc_grey(in)                getbitfield(in + 0x60, 1, 4)
#define get_inquiry_sc_gray(in)                getbitfield(in + 0x60, 1, 4)
#define get_inquiry_sc_color(in)               getbitfield(in + 0x60, 1, 5)
#define get_inquiry_sc_uta(in)                 getbitfield(in + 0x60, 1, 6)
#define get_inquiry_sc_adf(in)                 getbitfield(in + 0x60, 1, 7)
#define get_inquiry_sc_double_res(in)          getbitfield(in + 0x61, 1, 0)
#define get_inquiry_sc_high_byte_first(in)     getbitfield(in + 0x61, 1, 1)
#define get_inquiry_sc_bi_image_reverse(in)    getbitfield(in + 0x61, 1, 2)
#define get_inquiry_sc_multi_image_reverse(in) getbitfield(in + 0x61, 1, 3)
#define get_inquiry_sc_no_shadow(in)           getbitfield(in + 0x61, 1, 4)
#define get_inquiry_sc_no_highlight(in)        getbitfield(in + 0x61, 1, 5)
#define get_inquiry_sc_downloadable_fw(in)     getbitfield(in + 0x61, 1, 6)
#define get_inquiry_sc_uploadable_shade(in)    getbitfield(in + 0x62, 1, 0)
#define get_inquiry_gamma_available(in)      getbitfield(in + 0x64, 1, 7)
#define get_inquiry_gamma_type(in)           getbitfield(in + 0x64, 1, 5)
#define get_inquiry_gamma_lines(in)          getbitfield(in + 0x64, 0x03, 0)
#  define IN_3_pass 0x01
#  define IN_3_line 0x03
#define get_inquiry_gib(in) in[0x66]
#  define IN_gib_8bpp  0x01
#  define IN_gib_10bpp 0x04
#define get_inquiry_gob(in) in[0x68]
#  define IN_gob_8bpp  0x01
#  define IN_gob_10bpp 0x04
#define get_inquiry_hda(in) getbitfield(in + 0x69, 0x01, 7)
#define get_inquiry_max_halftone_matrix(in) getbitfield(in + 0x69, 0x7f, 0)
#define get_inquiry_halftones_supported(in) in[6a]
#  define IN_halftone_2x2   0x01
#  define IN_halftone_4x4   0x02
#  define IN_halftone_6x6   0x04
#  define IN_halftone_8x8   0x08
#  define IN_halftone_12x12 0x20
#define get_inquiry_colorseq(in) getbitfield(in + 0x6d, 0x07, 5)
#  define IN_color_sequence_RGB 0x00
#  define IN_color_sequence_RBG 0x01
#  define IN_color_sequence_GBR 0x02
#  define IN_color_sequence_GRB 0x03
#  define IN_color_sequence_BRG 0x04
#  define IN_color_sequence_BGR 0x05
#  define IN_color_sequence_all 0x07
#define get_inquiry_color_order(in) getbitfield(in + 0x6d, 0x1f, 0)
#  define IN_color_ordering_pixel       0x01
#  define IN_color_ordering_line_no_ccd 0x02
#  define IN_color_ordering_plane       0x04
#  define IN_color_ordering_line_w_ccd  0x08
#define get_inquiry_max_vidmem(in) getnbyte(in + 0x6e, 4)
#define get_inquiry_max_opt_res(in) in[0x73]
#define get_inquiry_max_x_res(in) in[0x74]
#define get_inquiry_max_y_res(in) in[0x75]
#define get_inquiry_fb_max_scan_width(in) getnbyte(in + 0x76, 2)
#define get_inquiry_fb_max_scan_length(in) getnbyte(in + 0x78, 2)
#define get_inquiry_uta_x_original(in) getnbyte(in + 0x7a, 2)
#define get_inquiry_uta_y_original(in) getnbyte(in + 0x7c, 2)
#define get_inquiry_uta_max_scan_width(in) getnbyte(in + 0x7e, 2)
#define get_inquiry_uta_max_scan_length(in) getnbyte(in + 0x80, 2)
#define get_inquiry_dor_x_original_point(in) getnbyte(in + 0x86, 2)
#define get_inquiry_dor_y_original_point(in) getnbyte(in + 0x88, 2)
#define get_inquiry_dor_max_scan_width(in) getnbyte(in + 0x8a, 2)
#define get_inquiry_dor_max_scan_length(in) getnbyte(in + 0x8c, 2)
#define get_inquiry_last_calibration_lamp_density(in) in[0x8f]
#  define IN_min_lamp_density 0x01
#  define IN_max_lamp_density 0xff
#define get_inquiry_lamp_warmup_maximum_time(in) in[0x91]
#define get_inquiry_wdb_length(in) getnbyte(in + 0x92, 2)


static unsigned char test_unit_readyC[] = {
  TEST_UNIT_READY, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk test_unit_ready = { test_unit_readyC, sizeof(test_unit_readyC) };

static unsigned char reserve_unitC[] = {
  RESERVE_UNIT, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk reserve_unit = { reserve_unitC, sizeof(reserve_unitC) };

static unsigned char release_unitC[] = {
  RELEASE_UNIT, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk release_unit = { release_unitC, sizeof(release_unitC) };

static unsigned char set_windowC[] = {
  SET_WINDOW, 0x00,       /* opcode, lun */
  0x00, 0x00, 0x00, 0x00, /* reserved */
  0x00, 0x00, 0x00,       /* transfer length; needs to be set */
#define set_SW_xferlen(sb, len) putnbyte(sb + 0x06, len, 3)
  0x00,                   /* control byte */
};
static scsiblk set_window = { set_windowC, sizeof(set_windowC) };
static unsigned char window_parameter_data_blockC[] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00,    /* reserved */
  0x00, 0x00,                            /* Window Descriptor Length,
                                            value must be set by
                                            SHORT_WDB define */
#define set_WPDB_wdblen(sb, len) putnbyte(sb + 0x06, len, 2)
};
static scsiblk window_parameter_data_block = {
  window_parameter_data_blockC, sizeof(window_parameter_data_blockC)
};
static unsigned char window_descriptor_blockC[] = {
  0x00,               /* Window Identifier */
#define set_WD_wid(sb, val) sb[0] = val
#    define WD_wid_bw    0x00
#    define WD_wid_red   0x01
#    define WD_wid_green 0x02
#    define WD_wid_blue  0x03
#    define WD_wid_all   0x00
  0x00,               /* reserved */
  0x00, 0x00,         /* X Resolution in dpi */
#define set_WD_Xres(sb, val) putnbyte(sb + 0x02, val, 2)
  0x00, 0x00,         /* Y Resolution in dpi */
#define set_WD_Yres(sb, val) putnbyte(sb + 0x04, val, 2)
  0x00, 0x00, 0x00, 0x00,
                      /* Upper Left X in 1200pt/inch */
#define set_WD_ULX(sb, val) putnbyte(sb + 0x06, val, 4)
  0x00, 0x00, 0x00, 0x00,
                      /* Upper Left Y in 1200pt/inch */
#define set_WD_ULY(sb, val) putnbyte(sb + 0x0a, val, 4)
  0x00, 0x00, 0x00, 0x00,
                      /* Width 1200pt/inch */
#define set_WD_width(sb, val) putnbyte(sb + 0x0e, val, 4)
  0x00, 0x00, 0x00, 0x00,
                      /* Length 1200pt/inch */
#define set_WD_length(sb, val) putnbyte(sb + 0x12, val, 4)
  0xff,               /* Brightness */
#define set_WD_brightness(sb, val) sb[0x16] = val
  0x00,               /* Threshold */
#define set_WD_threshold(sb, val) sb[0x17] = val
  0x80,               /* Contrast */
#define set_WD_contrast(sb, val) sb[0x18] = val
  0x05,               /* Image Composition */
#define set_WD_composition(sb, val) sb[0x19] = val
#define     WD_comp_lineart      0x00
#define     WD_comp_dithered     0x01
#define     WD_comp_grey         0x02
#define     WD_comp_gray         0x02
#define     WD_comp_rgb_bilevel  0x03
#define     WD_comp_rgb_dithered 0x04
#define     WD_comp_rgb_full     0x05
  0x08,               /* Bits/Pixel */
#define set_WD_bitsperpixel(sb, val) sb[0x1a] = val
#    define WD_bits_1  0x01
#    define WD_bits_8  0x08
#    define WD_bits_10 0x0a
  0x00, 0x03,         /* Halftone pattern
		       * Originally assumed 0x01; analyzer
		       * provided the value 0x03 */
#define set_WD_halftone(sb, val) putnbyte(sb + 0x1b, val, 2)
     /* not sure how to do this yet... */
  0x03,               /* RIF, reserved, padding type */
#define set_WD_RIF(sb, val) setbitfield(sb + 0x1d, 1, 7, val)
#define set_WD_padding_type(sb, val) setbitfield(sb + 0x1d, 0x07, 0, val)
#    define WD_padding_byte 0x03
#    define WD_padding_word 0x07
  0x00, 0x00,         /* Bit Ordering (0 = left to right; reserved) */
  0x00,               /* Compression Type (reserved) */
  0x00,               /* Compression Argument (reserved) */
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      /* Reserved */
  0x01,               /* Speed */
#define set_WD_speed(sb, val) sb[0x28] = val
#    define WD_speed_fastsmear 0x81
#    define WD_speed_slowsmear 0x82
#    define WD_speed_fastclear 0x01
#    define WD_speed_slowclear 0x02
  0x00,               /* Select Color */
#define set_WD_select_color(sb, val) setbitfield(sb + 0x29, 0x07, 5, val)
#    define WD_color_grey  0x00
#    define WD_color_gray  0x00
#    define WD_color_red   0x04
#    define WD_color_green 0x02
#    define WD_color_blue  0x01
  0xff,               /* Highlight */
#define set_WD_highlight(sb, val) sb[0x2a] = val
  0x00,               /* Shadow */
#define set_WD_shadow(sb, val) sb[0x2b] = val
  0x00, 0x00,         /* Paper Length */
#define set_WD_paperlength(sb, val) putnbyte(sb + 0x2c, val, 2)
  0x0f,               /* Gamma Function (default here is normal function) */
#define set_WD_gamma(sb, val) sb[0x2e] = val
#    define WD_gamma_download 0x00
#    define WD_gamma_builtin1 0x01
#    define WD_gamma_builtin2 0x02
#    define WD_gamma_builtin3 0x03
#    define WD_gamma_builtin4 0x04
#    define WD_gamma_normal   0x0f
  0x11,               /* Scan Module */
#define set_WD_module(sb, val) sb[0x2f] = val
#    define WD_module_flatbed      0x11
#    define WD_module_transparency 0xff

#ifdef LONG_WDB       /* Their software doesn't use the rest */
  0x01,               /* HBT, DOR, reserved, RMIF, CBHS Type */
                      /* CBHS = Contrast, Brightness, Highlight, Shadow */
#define set_WD_CBHS(sb, val) setbitfield(sb + 0x30, 0x3, 0, val)
#    define WD_CBHS_50  0x00
#    define WD_CBHS_255 0x01
                      /* RMIF = Reverse Multilevel Image Format */
#define set_WD_RMIF(sb, val) setbitfield(sb + 0x30, 1, 2, val)
                      /* DOR = Double Optical Resolution */
#define set_WD_DOR(sb, val) setbitfield(sb + 0x30, 1, 6, val)
                      /* HBT = High Byte Transfer: true means
		       * multi-byte pixels come high byte first */
#define set_WD_HBT(sb, val) setbitfield(sb + 0x30, 1, 7, val)
#    define WD_HBT_HBF 0
#    define WD_HBT_LBF 1
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      /* Reserved */
  0x00,               /* (For Autoexposure) */
  0x00,               /* BS, reserved, Calibration Mode */
#define set_WD_batch(sb, val) setbitfield(sb + 0x39, 1, 7, val)
#define set_WD_calibration(sb, val) setbitfield(sb + 0x39, 0x0f, 0, val)
#    define WD_calibration_image   0x00
#    define WD_calibration_lineart 0x0f
#    define WD_calibration_dither  0x0e
#    define WD_calibration_grey    0x0d
#    define WD_calibration_gray    0x0d
#    define WD_calibration_rgb     0x0a
#    define WD_calibration_ignore  0x09
  0x01,               /* Color Sequence, Color Ordering Support */
#define set_WD_color_sequence(sb, val) setbitfield(sb + 0x3a, 0x07, 5, val)
#    define WD_color_sequence_RGB 0x00
#    define WD_color_sequence_RBG 0x01
#    define WD_color_sequence_GBR 0x02
#    define WD_color_sequence_GRB 0x03
#    define WD_color_sequence_BRG 0x04
#    define WD_color_sequence_BGR 0x05
#define set_WD_color_ordering(sb, val) setbitfield(sb + 0x3a, 0x1f, 0, val)
#    define WD_color_ordering_pixel       0x01
#    define WD_color_ordering_line_no_ccd 0x02
#    define WD_color_ordering_plane       0x04
#    define WD_color_ordering_line_w_ccd  0x08
  0x00, 0x00,         /* Reserved */
  0x00,               /* Next Calibration Lamp Density */
#define set_WD_lamp_c_density(sb, val) sb[0x3d] = val
#    define WD_lamp_c_density_auto 0x00
#    define WD_lamp_c_density_min  0x01
#    define WD_lamp_c_density_max  0xff
  0x00,               /* Reserved */
  0x00,               /* Next Scan Lamp Density */
#define set_WD_lamp_s_density(sb, val) sb[0x3f] = val
#    define WD_lamp_s_density_auto 0x00
#    define WD_lamp_s_density_min  0x01
#    define WD_lamp_s_density_max  0xff
  0x00, 0x00, 0x00, 0x00,
                      /* Next Upper Left Y in 1200 dpt/inch */
#define set_WD_next_upper_left(sb, val) putnbyte(sb + 0x40, val, 4)
  0x00, 0x00, 0x00, 0x00,
                      /* Pixel Count */
#define set_WD_pixel_count(sb, val) putnbyte(sb + 0x44, val, 4)
  0x00, 0x00, 0x00, 0x00,
                      /* Line Count */
#define set_WD_line_count(sb, val) putnbyte(sb + 0x48, val, 4)
#endif /* defined LONG_WDB */
};
static scsiblk window_descriptor_block = {
  window_descriptor_blockC, sizeof(window_descriptor_blockC)
};
#define WPDB_OFF(b) (b + set_window.size)
#define WDB_OFF(b, n) (b + set_window.size + \
		       window_parameter_data_block.size + \
		       ( window_descriptor_block.size * (n - 1) ) )
#define set_WPDB_wdbnum(sb, n) set_WPDB_wdblen(sb, window_descriptor_block.size * n)


static unsigned char scanC[] = {
  SCAN, 0x00, 0x00, 0x00, 0x00, 0x20,
};
static scsiblk scan = { scanC, sizeof(scanC) };
#define set_SC_xfer_length(sb, val) sb[0x04] = val
#define set_SC_quality(sb, val) setbitfield(sb + 0x05, 1, 5, val)
#define set_SC_adf(sb, val) setbitfield(sb + 0x05, 1, 6, val)
#define set_SC_preview(sb, val) setbitfield(sb + 0x05, 1, 7, val)
#define set_SC_wid(sb, n, val) sb[0x05 + n] = val

/* sread instead of read because read is a libc primitive */
static unsigned char sreadC[] = {
  READ, 0x00,
  0x00, /* Data Type Code */
  0x00, /* reserved */
  0x00, 0x00, /* data type qualifier */
  0x00, 0x00, 0x00, /* transfer length */
  0x00 /* control */
};
static scsiblk sread = { sreadC, sizeof(sreadC) };
#define set_R_datatype_code(sb, val) sb[0x02] = val
#    define R_datatype_imagedata 0x00
#    define R_datatype_vendorspecific 0x01
#    define R_datatype_halftone 0x02
#    define R_datatype_gamma 0x03
#    define R_datatype_shading 0x80
#    define R_datatype_gain 0x81
#define set_R_datatype_qual_winid(sb, val) sb[0x05] = val
#define set_R_xfer_length(sb, val) putnbyte(sb + 0x06, val, 3)

static unsigned char sendC[] = {
  SEND, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk send = { sendC, sizeof(sendC) };
#define set_S_datatype_code(sb, val) sb[0x02] = val
#    define S_datatype_imagedata 0x00
#    define S_datatype_vendorspecific 0x01
#    define S_datatype_halftone 0x02
#    define S_datatype_gamma 0x03
#    define S_datatype_shading 0x80
#    define S_datatype_gain 0x81
#define set_S_datatype_qual_winid(sb, val) sb[0x05] = val
#define set_S_xfer_length(sb, val) putnbyte(sb + 0x06, val, 3)
#    define S_xfer_length_gamma(l) ((l * (1024 + 1)) + 1)
#    define S_xfer_length_1line 0x0402
#    define S_xfer_length_2line 0x0803
#    define S_xfer_length_3line 0x0c04

/* FIXME: add Type 0 download curve format */
/* FIXME: add Type 1 download curve format */
/* FIXME: add Type 2 download curve format */

static unsigned char object_positionC[] = {
  OBJECT_POSITION, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk object_position = {
  object_positionC, sizeof(object_positionC)
};

static unsigned char get_data_buffer_statusC[] = {
  GET_DATA_BUFFER_STATUS, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x0c, /* allocation length */
  0x00
};
static scsiblk get_data_buffer_status = {
  get_data_buffer_statusC, sizeof(get_data_buffer_statusC)
};
#define set_GDBS_wait(sb, val) setbitfield(sb + 0x01, 1, 0, val)
#define set_GDBS_len(sb, nwins) putnbyte(sb + 0x07, (4 + (nwins * 8)), 2)
/* DBS is Data Buffer Status, the header that is returned */
#define get_DBS_data_buffer_status_length(sb) getnbyte(sb, 3)
/* DBSD are the Data Buffer Status Descriptors, the records that
 * are returned following the DBS header. */
/* order is the number of the DBSD, from 1 to (length/8) */
#define get_DBSD_winid(sb, order) sb[4 + ((order - 1) * 8)]
#define get_DBSD_available_data_buffer(sb, order) getnbyte(sb + 4 + ((order - 1) * 8) + 2, 3)
#define get_DBSD_filled_data_buffer(sb, order) getnbyte(sb + 4 + ((order - 1) * 8) + 5, 3)

static unsigned char write_bufferC[] = {
  WRITE_BUFFER, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
static scsiblk write_buffer = {
  write_bufferC, sizeof(write_bufferC)
};
/* FIXME: spec seems incomplete at this point */

/* defines for request sense return block */
#define RS_filemark(b) getbitfield(b + 0x02, 1, 7)
#define RS_EOM(b) getbitfield(b + 0x02, 1, 6)
#define RS_ILI(b) getbitfield(b + 0x02, 1, 5)
#define RS_sense_key(b) getbitfield(b + 0x02, 0x0f, 0) /* right? */
/* need more documentation to figure out all return values */

#endif /* UMAX_UMAX_H */
